@@ -1,4 +1,4 @@
-# Javascript template for HTMLWriter
+# JavaScript template for HTMLWriter
 JS_INCLUDE = """
 <link rel="stylesheet"
 href="https://maxcdn.bootstrapcdn.com/font-awesome/4.4.0/css/font-awesome.min.css">
@@ -251,7 +251,7 @@ DISPLAY_TEMPLATE = """
     }}, 0);
   }})()
 </script>
-"""
+"""  # noqa: E501
 
 
 INCLUDED_FRAMES = """
