@@ -7,7 +7,7 @@ from matplotlib import pyplot as plt
 from matplotlib.testing.decorators import image_comparison, check_figures_equal
 
 
-@image_comparison(['polar_axes'], style='default', tol=0.012)
+@image_comparison(['polar_axes.png'], style='default', tol=0.012)
 def test_polar_annotations():
     # You can specify the xypoint and the xytext in different positions and
     # coordinate systems, and optionally turn on a connecting line and mark the
@@ -41,8 +41,8 @@ def test_polar_annotations():
     ax.tick_params(axis='x', tick1On=True, tick2On=True, direction='out')
 
 
-@image_comparison(['polar_coords'], style='default', remove_text=True,
-                  tol=0.012)
+@image_comparison(['polar_coords.png'], style='default', remove_text=True,
+                  tol=0.014)
 def test_polar_coord_annotations():
     # You can also use polar notation on a cartesian axes.  Here the native
     # coordinate system ('data') is cartesian, so you need to specify the
@@ -95,10 +95,10 @@ def test_polar_twice():
     fig = plt.figure()
     plt.polar([1, 2], [.1, .2])
     plt.polar([3, 4], [.3, .4])
-    assert len(fig.axes) == 1, 'More than one polar axes created.'
+    assert len(fig.axes) == 1, 'More than one polar Axes created.'
 
 
-@check_figures_equal()
+@check_figures_equal(extensions=['png'])
 def test_polar_wrap(fig_test, fig_ref):
     ax = fig_test.add_subplot(projection="polar")
     ax.plot(np.deg2rad([179, -179]), [0.2, 0.1])
@@ -108,7 +108,7 @@ def test_polar_wrap(fig_test, fig_ref):
     ax.plot(np.deg2rad([2, 358]), [0.2, 0.1])
 
 
-@check_figures_equal()
+@check_figures_equal(extensions=['png'])
 def test_polar_units_1(fig_test, fig_ref):
     import matplotlib.testing.jpl_units as units
     units.register()
@@ -123,7 +123,7 @@ def test_polar_units_1(fig_test, fig_ref):
     ax.set(xlabel="deg")
 
 
-@check_figures_equal()
+@check_figures_equal(extensions=['png'])
 def test_polar_units_2(fig_test, fig_ref):
     import matplotlib.testing.jpl_units as units
     units.register()
@@ -144,7 +144,7 @@ def test_polar_units_2(fig_test, fig_ref):
     ax.set(xlabel="rad", ylabel="km")
 
 
-@image_comparison(['polar_rmin'], style='default')
+@image_comparison(['polar_rmin.png'], style='default')
 def test_polar_rmin():
     r = np.arange(0, 3.0, 0.01)
     theta = 2*np.pi*r
@@ -156,7 +156,7 @@ def test_polar_rmin():
     ax.set_rmin(0.5)
 
 
-@image_comparison(['polar_negative_rmin'], style='default')
+@image_comparison(['polar_negative_rmin.png'], style='default')
 def test_polar_negative_rmin():
     r = np.arange(-3.0, 0.0, 0.01)
     theta = 2*np.pi*r
@@ -168,7 +168,7 @@ def test_polar_negative_rmin():
     ax.set_rmin(-3.0)
 
 
-@image_comparison(['polar_rorigin'], style='default')
+@image_comparison(['polar_rorigin.png'], style='default')
 def test_polar_rorigin():
     r = np.arange(0, 3.0, 0.01)
     theta = 2*np.pi*r
@@ -200,7 +200,7 @@ def test_polar_invertedylim_rorigin():
     ax.set_rorigin(3)
 
 
-@image_comparison(['polar_theta_position'], style='default')
+@image_comparison(['polar_theta_position.png'], style='default')
 def test_polar_theta_position():
     r = np.arange(0, 3.0, 0.01)
     theta = 2*np.pi*r
@@ -212,7 +212,7 @@ def test_polar_theta_position():
     ax.set_theta_direction('clockwise')
 
 
-@image_comparison(['polar_rlabel_position'], style='default')
+@image_comparison(['polar_rlabel_position.png'], style='default')
 def test_polar_rlabel_position():
     fig = plt.figure()
     ax = fig.add_subplot(projection='polar')
@@ -220,7 +220,14 @@ def test_polar_rlabel_position():
     ax.tick_params(rotation='auto')
 
 
-@image_comparison(['polar_theta_wedge'], style='default')
+@image_comparison(['polar_title_position.png'], style='mpl20')
+def test_polar_title_position():
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='polar')
+    ax.set_title('foo')
+
+
+@image_comparison(['polar_theta_wedge.png'], style='default')
 def test_polar_theta_limits():
     r = np.arange(0, 3.0, 0.01)
     theta = 2*np.pi*r
@@ -291,6 +298,13 @@ def test_polar_no_data():
     assert ax.get_rmin() == 0 and ax.get_rmax() == 1
 
 
+def test_polar_default_log_lims():
+    plt.subplot(projection='polar')
+    ax = plt.gca()
+    ax.set_rscale('log')
+    assert ax.get_rmin() > 0
+
+
 def test_polar_not_datalim_adjustable():
     ax = plt.figure().add_subplot(projection="polar")
     with pytest.raises(ValueError):
@@ -418,12 +432,97 @@ def test_axvline_axvspan_do_not_modify_rlims():
 def test_cursor_precision():
     ax = plt.subplot(projection="polar")
     # Higher radii correspond to higher theta-precisions.
-    assert ax.format_coord(0, 0) == "θ=0π (0°), r=0.000"
+    assert ax.format_coord(0, 0.005) == "θ=0.0π (0°), r=0.005"
     assert ax.format_coord(0, .1) == "θ=0.00π (0°), r=0.100"
     assert ax.format_coord(0, 1) == "θ=0.000π (0.0°), r=1.000"
-    assert ax.format_coord(1, 0) == "θ=0.3π (57°), r=0.000"
+    assert ax.format_coord(1, 0.005) == "θ=0.3π (57°), r=0.005"
     assert ax.format_coord(1, .1) == "θ=0.32π (57°), r=0.100"
     assert ax.format_coord(1, 1) == "θ=0.318π (57.3°), r=1.000"
-    assert ax.format_coord(2, 0) == "θ=0.6π (115°), r=0.000"
+    assert ax.format_coord(2, 0.005) == "θ=0.6π (115°), r=0.005"
     assert ax.format_coord(2, .1) == "θ=0.64π (115°), r=0.100"
     assert ax.format_coord(2, 1) == "θ=0.637π (114.6°), r=1.000"
+
+
+def test_custom_fmt_data():
+    ax = plt.subplot(projection="polar")
+    def millions(x):
+        return '$%1.1fM' % (x*1e-6)
+
+    # Test only x formatter
+    ax.fmt_xdata = None
+    ax.fmt_ydata = millions
+    assert ax.format_coord(12, 2e7) == "θ=3.8197186342π (687.54935416°), r=$20.0M"
+    assert ax.format_coord(1234, 2e6) == "θ=392.794399551π (70702.9919191°), r=$2.0M"
+    assert ax.format_coord(3, 100) == "θ=0.95493π (171.887°), r=$0.0M"
+
+    # Test only y formatter
+    ax.fmt_xdata = millions
+    ax.fmt_ydata = None
+    assert ax.format_coord(2e5, 1) == "θ=$0.2M, r=1.000"
+    assert ax.format_coord(1, .1) == "θ=$0.0M, r=0.100"
+    assert ax.format_coord(1e6, 0.005) == "θ=$1.0M, r=0.005"
+
+    # Test both x and y formatters
+    ax.fmt_xdata = millions
+    ax.fmt_ydata = millions
+    assert ax.format_coord(2e6, 2e4*3e5) == "θ=$2.0M, r=$6000.0M"
+    assert ax.format_coord(1e18, 12891328123) == "θ=$1000000000000.0M, r=$12891.3M"
+    assert ax.format_coord(63**7, 1081968*1024) == "θ=$3938980.6M, r=$1107.9M"
+
+
+@image_comparison(['polar_log.png'], style='default')
+def test_polar_log():
+    fig = plt.figure()
+    ax = fig.add_subplot(polar=True)
+
+    ax.set_rscale('log')
+    ax.set_rlim(1, 1000)
+
+    n = 100
+    ax.plot(np.linspace(0, 2 * np.pi, n), np.logspace(0, 2, n))
+
+
+@check_figures_equal()
+def test_polar_log_rorigin(fig_ref, fig_test):
+    # Test that equivalent linear and log radial settings give the same axes patch
+    # and spines.
+    ax_ref = fig_ref.add_subplot(projection='polar', facecolor='red')
+    ax_ref.set_rlim(0, 2)
+    ax_ref.set_rorigin(-3)
+    ax_ref.set_rticks(np.linspace(0, 2, 5))
+
+    ax_test = fig_test.add_subplot(projection='polar', facecolor='red')
+    ax_test.set_rscale('log')
+    ax_test.set_rlim(1, 100)
+    ax_test.set_rorigin(10**-3)
+    ax_test.set_rticks(np.logspace(0, 2, 5))
+
+    for ax in ax_ref, ax_test:
+        # Radial tick labels should be the only difference, so turn them off.
+        ax.tick_params(labelleft=False)
+
+
+def test_polar_neg_theta_lims():
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='polar')
+    ax.set_thetalim(-np.pi, np.pi)
+    labels = [l.get_text() for l in ax.xaxis.get_ticklabels()]
+    assert labels == ['-180°', '-135°', '-90°', '-45°', '0°', '45°', '90°', '135°']
+
+
+@pytest.mark.parametrize("order", ["before", "after"])
+@image_comparison(baseline_images=['polar_errorbar.png'], remove_text=True,
+                  style='mpl20')
+def test_polar_errorbar(order):
+    theta = np.arange(0, 2 * np.pi, np.pi / 8)
+    r = theta / np.pi / 2 + 0.5
+    fig = plt.figure(figsize=(5, 5))
+    ax = fig.add_subplot(projection='polar')
+    if order == "before":
+        ax.set_theta_zero_location("N")
+        ax.set_theta_direction(-1)
+        ax.errorbar(theta, r, xerr=0.1, yerr=0.1, capsize=7, fmt="o", c="seagreen")
+    else:
+        ax.errorbar(theta, r, xerr=0.1, yerr=0.1, capsize=7, fmt="o", c="seagreen")
+        ax.set_theta_zero_location("N")
+        ax.set_theta_direction(-1)
