@@ -6,15 +6,19 @@
 #ifndef MPL_BACKEND_AGG_H
 #define MPL_BACKEND_AGG_H
 
+#include <pybind11/pybind11.h>
+
 #include <cmath>
-#include <vector>
 #include <algorithm>
+#include <functional>
 
 #include "agg_alpha_mask_u8.h"
 #include "agg_conv_curve.h"
 #include "agg_conv_dash.h"
 #include "agg_conv_stroke.h"
+#include "agg_conv_transform.h"
 #include "agg_image_accessors.h"
+#include "agg_path_storage.h"
 #include "agg_pixfmt_amask_adaptor.h"
 #include "agg_pixfmt_gray.h"
 #include "agg_pixfmt_rgba.h"
@@ -25,7 +29,6 @@
 #include "agg_scanline_bin.h"
 #include "agg_scanline_p.h"
 #include "agg_scanline_storage_aa.h"
-#include "agg_scanline_storage_bin.h"
 #include "agg_scanline_u.h"
 #include "agg_span_allocator.h"
 #include "agg_span_converter.h"
@@ -34,16 +37,17 @@
 #include "agg_span_image_filter_rgba.h"
 #include "agg_span_interpolator_linear.h"
 #include "agg_span_pattern_rgba.h"
-#include "util/agg_color_conv_rgb8.h"
 
 #include "_backend_agg_basic_types.h"
 #include "path_converters.h"
 #include "array.h"
 #include "agg_workaround.h"
 
+namespace py = pybind11;
+
 /**********************************************************************/
 
-// a helper class to pass agg::buffer objects around.  
+// a helper class to pass agg::buffer objects around.
 
 class BufferRegion
 {
@@ -86,8 +90,6 @@ class BufferRegion
         return stride;
     }
 
-    void to_string_argb(uint8_t *buf);
-
   private:
     agg::int8u *data;
     agg::rect_i rect;
@@ -115,10 +117,10 @@ class RendererAgg
     typedef agg::renderer_scanline_bin_solid<renderer_base> renderer_bin;
     typedef agg::rasterizer_scanline_aa<agg::rasterizer_sl_clip_dbl> rasterizer;
 
-    typedef agg::scanline_p8 scanline_p8;
-    typedef agg::scanline_bin scanline_bin;
+    typedef agg::scanline32_p8 scanline_p8;
+    typedef agg::scanline32_bin scanline_bin;
     typedef agg::amask_no_clip_gray8 alpha_mask_type;
-    typedef agg::scanline_u8_am<alpha_mask_type> scanline_am;
+    typedef agg::scanline32_u8_am<alpha_mask_type> scanline_am;
 
     typedef agg::renderer_base<agg::pixfmt_gray8> renderer_base_alpha_mask_type;
     typedef agg::renderer_scanline_aa_solid<renderer_base_alpha_mask_type> renderer_alpha_mask_type;
@@ -190,12 +192,6 @@ class RendererAgg
                         bool antialiased,
                         ColorArray &edgecolors);
 
-    template <class PointArray, class ColorArray>
-    void draw_gouraud_triangle(GCAgg &gc,
-                               PointArray &points,
-                               ColorArray &colors,
-                               agg::trans_affine &trans);
-
     template <class PointArray, class ColorArray>
     void draw_gouraud_triangles(GCAgg &gc,
                                 PointArray &points,
@@ -250,7 +246,7 @@ class RendererAgg
     template <class R>
     void set_clipbox(const agg::rect_d &cliprect, R &rasterizer);
 
-    bool render_clippath(py::PathIterator &clippath, const agg::trans_affine &clippath_trans, e_snap_mode snap_mode);
+    bool render_clippath(mpl::PathIterator &clippath, const agg::trans_affine &clippath_trans, e_snap_mode snap_mode);
 
     template <class PathIteratorType>
     void _draw_path(PathIteratorType &path, bool has_clippath, const facepair_t &face, GCAgg &gc);
@@ -346,15 +342,16 @@ RendererAgg::_draw_path(path_t &path, bool has_clippath, const facepair_t &face,
         rendererBase.reset_clipping(true);
 
         // Create and transform the path
-        typedef agg::conv_transform<py::PathIterator> hatch_path_trans_t;
+        typedef agg::conv_transform<mpl::PathIterator> hatch_path_trans_t;
         typedef agg::conv_curve<hatch_path_trans_t> hatch_path_curve_t;
         typedef agg::conv_stroke<hatch_path_curve_t> hatch_path_stroke_t;
 
-        py::PathIterator hatch_path(gc.hatchpath);
+        mpl::PathIterator hatch_path(gc.hatchpath);
         agg::trans_affine hatch_trans;
         hatch_trans *= agg::trans_affine_scaling(1.0, -1.0);
         hatch_trans *= agg::trans_affine_translation(0.0, 1.0);
-        hatch_trans *= agg::trans_affine_scaling(hatch_size, hatch_size);
+        hatch_trans *= agg::trans_affine_scaling(static_cast<double>(hatch_size),
+                                                 static_cast<double>(hatch_size));
         hatch_path_trans_t hatch_path_trans(hatch_path, hatch_trans);
         hatch_path_curve_t hatch_path_curve(hatch_path_trans);
         hatch_path_stroke_t hatch_path_stroke(hatch_path_curve);
@@ -453,7 +450,7 @@ template <class PathIterator>
 inline void
 RendererAgg::draw_path(GCAgg &gc, PathIterator &path, agg::trans_affine &trans, agg::rgba &color)
 {
-    typedef agg::conv_transform<py::PathIterator> transformed_path_t;
+    typedef agg::conv_transform<mpl::PathIterator> transformed_path_t;
     typedef PathNanRemover<transformed_path_t> nan_removed_t;
     typedef PathClipper<nan_removed_t> clipped_t;
     typedef PathSnapper<clipped_t> snapped_t;
@@ -496,7 +493,7 @@ inline void RendererAgg::draw_markers(GCAgg &gc,
                                       agg::trans_affine &trans,
                                       agg::rgba color)
 {
-    typedef agg::conv_transform<py::PathIterator> transformed_path_t;
+    typedef agg::conv_transform<mpl::PathIterator> transformed_path_t;
     typedef PathNanRemover<transformed_path_t> nan_removed_t;
     typedef PathSnapper<nan_removed_t> snap_t;
     typedef agg::conv_curve<snap_t> curve_t;
@@ -737,22 +734,25 @@ inline void RendererAgg::draw_text_image(GCAgg &gc, ImageArray &image, int x, in
     rendererBase.reset_clipping(true);
     if (angle != 0.0) {
         agg::rendering_buffer srcbuf(
-                image.data(), (unsigned)image.dim(1),
-                (unsigned)image.dim(0), (unsigned)image.dim(1));
+                image.mutable_data(0, 0), (unsigned)image.shape(1),
+                (unsigned)image.shape(0), (unsigned)image.shape(1));
         agg::pixfmt_gray8 pixf_img(srcbuf);
 
         set_clipbox(gc.cliprect, theRasterizer);
 
+        auto image_height = static_cast<double>(image.shape(0)),
+             image_width = static_cast<double>(image.shape(1));
+
         agg::trans_affine mtx;
-        mtx *= agg::trans_affine_translation(0, -image.dim(0));
-        mtx *= agg::trans_affine_rotation(-angle * agg::pi / 180.0);
+        mtx *= agg::trans_affine_translation(0, -image_height);
+        mtx *= agg::trans_affine_rotation(-angle * (agg::pi / 180.0));
         mtx *= agg::trans_affine_translation(x, y);
 
         agg::path_storage rect;
         rect.move_to(0, 0);
-        rect.line_to(image.dim(1), 0);
-        rect.line_to(image.dim(1), image.dim(0));
-        rect.line_to(0, image.dim(0));
+        rect.line_to(image_width, 0);
+        rect.line_to(image_width, image_height);
+        rect.line_to(0, image_height);
         rect.line_to(0, 0);
         agg::conv_transform<agg::path_storage> rect2(rect, mtx);
 
@@ -773,24 +773,28 @@ inline void RendererAgg::draw_text_image(GCAgg &gc, ImageArray &image, int x, in
     } else {
         agg::rect_i fig, text;
 
+        int deltay = y - image.shape(0);
+
         fig.init(0, 0, width, height);
-        text.init(x, y - image.dim(0), x + image.dim(1), y);
+        text.init(x, deltay, x + image.shape(1), y);
         text.clip(fig);
 
         if (gc.cliprect.x1 != 0.0 || gc.cliprect.y1 != 0.0 || gc.cliprect.x2 != 0.0 || gc.cliprect.y2 != 0.0) {
             agg::rect_i clip;
 
-            clip.init(int(mpl_round(gc.cliprect.x1)),
-                      int(mpl_round(height - gc.cliprect.y2)),
-                      int(mpl_round(gc.cliprect.x2)),
-                      int(mpl_round(height - gc.cliprect.y1)));
+            clip.init(mpl_round_to_int(gc.cliprect.x1),
+                      mpl_round_to_int(height - gc.cliprect.y2),
+                      mpl_round_to_int(gc.cliprect.x2),
+                      mpl_round_to_int(height - gc.cliprect.y1));
             text.clip(clip);
         }
 
         if (text.x2 > text.x1) {
+            int deltax = text.x2 - text.x1;
+            int deltax2 = text.x1 - x;
             for (int yi = text.y1; yi < text.y2; ++yi) {
-                pixFmt.blend_solid_hspan(text.x1, yi, (text.x2 - text.x1), gc.color,
-                                         &image(yi - (y - image.dim(0)), text.x1 - x));
+                pixFmt.blend_solid_hspan(text.x1, yi, deltax, gc.color,
+                                         &image(yi - deltay, deltax2));
             }
         }
     }
@@ -833,20 +837,24 @@ inline void RendererAgg::draw_image(GCAgg &gc,
     bool has_clippath = render_clippath(gc.clippath.path, gc.clippath.trans, gc.snap_mode);
 
     agg::rendering_buffer buffer;
-    buffer.attach(
-        image.data(), (unsigned)image.dim(1), (unsigned)image.dim(0), -(int)image.dim(1) * 4);
+    buffer.attach(image.mutable_data(0, 0, 0),
+                  (unsigned)image.shape(1), (unsigned)image.shape(0),
+                  -(int)image.shape(1) * 4);
     pixfmt pixf(buffer);
 
     if (has_clippath) {
         agg::trans_affine mtx;
         agg::path_storage rect;
 
-        mtx *= agg::trans_affine_translation((int)x, (int)(height - (y + image.dim(0))));
+        auto image_height = static_cast<double>(image.shape(0)),
+             image_width = static_cast<double>(image.shape(1));
+
+        mtx *= agg::trans_affine_translation((int)x, (int)(height - (y + image_height)));
 
         rect.move_to(0, 0);
-        rect.line_to(image.dim(1), 0);
-        rect.line_to(image.dim(1), image.dim(0));
-        rect.line_to(0, image.dim(0));
+        rect.line_to(image_width, 0);
+        rect.line_to(image_width, image_height);
+        rect.line_to(0, image_height);
         rect.line_to(0, 0);
 
         agg::conv_transform<agg::path_storage> rect2(rect, mtx);
@@ -882,7 +890,7 @@ inline void RendererAgg::draw_image(GCAgg &gc,
     } else {
         set_clipbox(gc.cliprect, rendererBase);
         rendererBase.blend_from(
-            pixf, 0, (int)x, (int)(height - (y + image.dim(0))), (agg::int8u)(alpha * 255));
+            pixf, 0, (int)x, (int)(height - (y + image.shape(0))), (agg::int8u)(alpha * 255));
     }
 
     rendererBase.reset_clipping(true);
@@ -918,17 +926,21 @@ inline void RendererAgg::_draw_path_collection_generic(GCAgg &gc,
     typedef PathSnapper<clipped_t> snapped_t;
     typedef agg::conv_curve<snapped_t> snapped_curve_t;
     typedef agg::conv_curve<clipped_t> curve_t;
+    typedef Sketch<clipped_t> sketch_clipped_t;
+    typedef Sketch<curve_t> sketch_curve_t;
+    typedef Sketch<snapped_t> sketch_snapped_t;
+    typedef Sketch<snapped_curve_t> sketch_snapped_curve_t;
 
     size_t Npaths = path_generator.num_paths();
-    size_t Noffsets = offsets.size();
+    size_t Noffsets = safe_first_shape(offsets);
     size_t N = std::max(Npaths, Noffsets);
 
-    size_t Ntransforms = transforms.size();
-    size_t Nfacecolors = facecolors.size();
-    size_t Nedgecolors = edgecolors.size();
-    size_t Nlinewidths = linewidths.size();
+    size_t Ntransforms = safe_first_shape(transforms);
+    size_t Nfacecolors = safe_first_shape(facecolors);
+    size_t Nedgecolors = safe_first_shape(edgecolors);
+    size_t Nlinewidths = safe_first_shape(linewidths);
     size_t Nlinestyles = std::min(linestyles.size(), N);
-    size_t Naa = antialiaseds.size();
+    size_t Naa = safe_first_shape(antialiaseds);
 
     if ((Nfacecolors == 0 && Nedgecolors == 0) || Npaths == 0) {
         return;
@@ -945,6 +957,7 @@ inline void RendererAgg::_draw_path_collection_generic(GCAgg &gc,
     facepair_t face;
     face.first = Nfacecolors != 0;
     agg::trans_affine trans;
+    bool do_clip = !face.first && !gc.has_hatchpath();
 
     for (int i = 0; i < (int)N; ++i) {
         typename PathGenerator::path_iterator path = path_generator(i);
@@ -992,33 +1005,29 @@ inline void RendererAgg::_draw_path_collection_generic(GCAgg &gc,
             }
         }
 
-        bool do_clip = !face.first && !gc.has_hatchpath();
-
+        gc.isaa = antialiaseds(i % Naa);
+        transformed_path_t tpath(path, trans);
+        nan_removed_t nan_removed(tpath, true, has_codes);
+        clipped_t clipped(nan_removed, do_clip, width, height);
         if (check_snap) {
-            gc.isaa = antialiaseds(i % Naa);
-
-            transformed_path_t tpath(path, trans);
-            nan_removed_t nan_removed(tpath, true, has_codes);
-            clipped_t clipped(nan_removed, do_clip, width, height);
             snapped_t snapped(
                 clipped, gc.snap_mode, path.total_vertices(), points_to_pixels(gc.linewidth));
             if (has_codes) {
                 snapped_curve_t curve(snapped);
-                _draw_path(curve, has_clippath, face, gc);
+                sketch_snapped_curve_t sketch(curve, gc.sketch.scale, gc.sketch.length, gc.sketch.randomness);
+                _draw_path(sketch, has_clippath, face, gc);
             } else {
-                _draw_path(snapped, has_clippath, face, gc);
+                sketch_snapped_t sketch(snapped, gc.sketch.scale, gc.sketch.length, gc.sketch.randomness);
+                _draw_path(sketch, has_clippath, face, gc);
             }
         } else {
-            gc.isaa = antialiaseds(i % Naa);
-
-            transformed_path_t tpath(path, trans);
-            nan_removed_t nan_removed(tpath, true, has_codes);
-            clipped_t clipped(nan_removed, do_clip, width, height);
             if (has_codes) {
                 curve_t curve(clipped);
-                _draw_path(curve, has_clippath, face, gc);
+                sketch_curve_t sketch(curve, gc.sketch.scale, gc.sketch.length, gc.sketch.randomness);
+                _draw_path(sketch, has_clippath, face, gc);
             } else {
-                _draw_path(clipped, has_clippath, face, gc);
+                sketch_clipped_t sketch(clipped, gc.sketch.scale, gc.sketch.length, gc.sketch.randomness);
+                _draw_path(sketch, has_clippath, face, gc);
             }
         }
     }
@@ -1124,7 +1133,7 @@ class QuadMeshGenerator
 
     inline size_t num_paths() const
     {
-        return m_meshWidth * m_meshHeight;
+        return (size_t) m_meshWidth * m_meshHeight;
     }
 
     inline path_iterator operator()(size_t i) const
@@ -1190,6 +1199,9 @@ inline void RendererAgg::_draw_gouraud_triangle(PointArray &points,
             tpoints[i][j] = points(i, j);
         }
         trans.transform(&tpoints[i][0], &tpoints[i][1]);
+        if(std::isnan(tpoints[i][0]) || std::isnan(tpoints[i][1])) {
+            return;
+        }
     }
 
     span_alloc_t span_alloc;
@@ -1223,34 +1235,33 @@ inline void RendererAgg::_draw_gouraud_triangle(PointArray &points,
     }
 }
 
-template <class PointArray, class ColorArray>
-inline void RendererAgg::draw_gouraud_triangle(GCAgg &gc,
-                                               PointArray &points,
-                                               ColorArray &colors,
-                                               agg::trans_affine &trans)
-{
-    theRasterizer.reset_clipping();
-    rendererBase.reset_clipping(true);
-    set_clipbox(gc.cliprect, theRasterizer);
-    bool has_clippath = render_clippath(gc.clippath.path, gc.clippath.trans, gc.snap_mode);
-
-    _draw_gouraud_triangle(points, colors, trans, has_clippath);
-}
-
 template <class PointArray, class ColorArray>
 inline void RendererAgg::draw_gouraud_triangles(GCAgg &gc,
                                                 PointArray &points,
                                                 ColorArray &colors,
                                                 agg::trans_affine &trans)
 {
+    if (points.shape(0)) {
+        check_trailing_shape(points, "points", 3, 2);
+    }
+    if (colors.shape(0)) {
+        check_trailing_shape(colors, "colors", 3, 4);
+    }
+    if (points.shape(0) != colors.shape(0)) {
+        throw py::value_error(
+            "points and colors arrays must be the same length, got " +
+            std::to_string(points.shape(0)) + " points and " +
+            std::to_string(colors.shape(0)) + "colors");
+    }
+
     theRasterizer.reset_clipping();
     rendererBase.reset_clipping(true);
     set_clipbox(gc.cliprect, theRasterizer);
     bool has_clippath = render_clippath(gc.clippath.path, gc.clippath.trans, gc.snap_mode);
 
-    for (int i = 0; i < points.dim(0); ++i) {
-        typename PointArray::sub_t point = points.subarray(i);
-        typename ColorArray::sub_t color = colors.subarray(i);
+    for (int i = 0; i < points.shape(0); ++i) {
+        auto point = std::bind(points, i, std::placeholders::_1, std::placeholders::_2);
+        auto color = std::bind(colors, i, std::placeholders::_1, std::placeholders::_2);
 
         _draw_gouraud_triangle(point, color, trans, has_clippath);
     }
