@@ -86,8 +86,8 @@ typedef struct Tk_PhotoImageBlock
     int offset[4];
 } Tk_PhotoImageBlock;
 
-#define TK_PHOTO_COMPOSITE_OVERLAY	0  // apply transparency rules pixel-wise
-#define TK_PHOTO_COMPOSITE_SET		1  // set image buffer directly
+#define TK_PHOTO_COMPOSITE_OVERLAY  0  // apply transparency rules pixel-wise
+#define TK_PHOTO_COMPOSITE_SET      1  // set image buffer directly
 #define TCL_OK     0
 #define TCL_ERROR  1
 
@@ -100,11 +100,13 @@ typedef int (*Tk_PhotoPutBlock_t) (Tcl_Interp *interp, Tk_PhotoHandle handle,
         Tk_PhotoImageBlock *blockPtr, int x, int y,
         int width, int height, int compRule);
 
-#ifdef WIN32_DLL
 /* Typedefs derived from function signatures in Tcl header */
+/* Tcl_SetVar typedef */
 typedef const char *(*Tcl_SetVar_t)(Tcl_Interp *interp, const char *varName,
                                     const char *newValue, int flags);
-#endif
+/* Tcl_SetVar2 typedef */
+typedef const char *(*Tcl_SetVar2_t)(Tcl_Interp *interp, const char *part1, const char *part2,
+                                     const char *newValue, int flags);
 
 #ifdef __cplusplus
 }
