@@ -1,10 +1,16 @@
 /* -*- mode: c++; c-basic-offset: 4 -*- */
 
 /* A python interface to FreeType */
+#pragma once
+
 #ifndef MPL_FT2FONT_H
 #define MPL_FT2FONT_H
+
+#include <set>
+#include <string>
+#include <string_view>
+#include <unordered_map>
 #include <vector>
-#include <stdint.h>
 
 extern "C" {
 #include <ft2build.h>
@@ -16,9 +22,6 @@ extern "C" {
 #include FT_TRUETYPE_TABLES_H
 }
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-
 /*
  By definition, FT_FIXED as 2 16bit values stored in a single long.
  */
@@ -35,8 +38,6 @@ class FT2Image
 
     void resize(long width, long height);
     void draw_bitmap(FT_Bitmap *bitmap, FT_Int x, FT_Int y);
-    void write_bitmap(FILE *fp) const;
-    void draw_rect(unsigned long x0, unsigned long y0, unsigned long x1, unsigned long y1);
     void draw_rect_filled(unsigned long x0, unsigned long y0, unsigned long x1, unsigned long y1);
 
     unsigned char *get_buffer()
@@ -53,7 +54,6 @@ class FT2Image
     }
 
   private:
-    bool m_dirty;
     unsigned char *m_buffer;
     unsigned long m_width;
     unsigned long m_height;
@@ -67,62 +67,85 @@ extern FT_Library _ft2Library;
 
 class FT2Font
 {
+    typedef void (*WarnFunc)(FT_ULong charcode, std::set<FT_String*> family_names);
 
   public:
-    FT2Font(FT_Open_Args &open_args, long hinting_factor);
+    FT2Font(FT_Open_Args &open_args, long hinting_factor,
+            std::vector<FT2Font *> &fallback_list, WarnFunc warn);
     virtual ~FT2Font();
     void clear();
     void set_size(double ptsize, double dpi);
     void set_charmap(int i);
     void select_charmap(unsigned long i);
-    void set_text(
-        size_t N, uint32_t *codepoints, double angle, FT_Int32 flags, std::vector<double> &xys);
-    int get_kerning(FT_UInt left, FT_UInt right, FT_UInt mode);
+    void set_text(std::u32string_view codepoints, double angle, FT_Int32 flags,
+                  std::vector<double> &xys);
+    int get_kerning(FT_UInt left, FT_UInt right, FT_Kerning_Mode mode, bool fallback);
+    int get_kerning(FT_UInt left, FT_UInt right, FT_Kerning_Mode mode, FT_Vector &delta);
     void set_kerning_factor(int factor);
-    void load_char(long charcode, FT_Int32 flags);
+    void load_char(long charcode, FT_Int32 flags, FT2Font *&ft_object, bool fallback);
+    bool load_char_with_fallback(FT2Font *&ft_object_with_glyph,
+                                 FT_UInt &final_glyph_index,
+                                 std::vector<FT_Glyph> &parent_glyphs,
+                                 std::unordered_map<long, FT2Font *> &parent_char_to_font,
+                                 std::unordered_map<FT_UInt, FT2Font *> &parent_glyph_to_font,
+                                 long charcode,
+                                 FT_Int32 flags,
+                                 FT_Error &charcode_error,
+                                 FT_Error &glyph_error,
+                                 std::set<FT_String*> &glyph_seen_fonts,
+                                 bool override);
+    void load_glyph(FT_UInt glyph_index, FT_Int32 flags, FT2Font *&ft_object, bool fallback);
     void load_glyph(FT_UInt glyph_index, FT_Int32 flags);
     void get_width_height(long *width, long *height);
     void get_bitmap_offset(long *x, long *y);
     long get_descent();
-    // TODO: Since we know the size of the array upfront, we probably don't
-    // need to dynamically allocate like this
-    void get_xys(bool antialiased, std::vector<double> &xys);
     void draw_glyphs_to_bitmap(bool antialiased);
     void draw_glyph_to_bitmap(FT2Image &im, int x, int y, size_t glyphInd, bool antialiased);
-    void get_glyph_name(unsigned int glyph_number, char *buffer);
+    void get_glyph_name(unsigned int glyph_number, std::string &buffer, bool fallback);
     long get_name_index(char *name);
-    PyObject* get_path();
+    FT_UInt get_char_index(FT_ULong charcode, bool fallback);
+    void get_path(std::vector<double> &vertices, std::vector<unsigned char> &codes);
+    bool get_char_fallback_index(FT_ULong charcode, int& index) const;
 
-    FT_Face &get_face()
+    FT_Face const &get_face() const
     {
         return face;
     }
+
     FT2Image &get_image()
     {
         return image;
     }
-    FT_Glyph &get_last_glyph()
+    FT_Glyph const &get_last_glyph() const
     {
         return glyphs.back();
     }
-    size_t get_last_glyph_index()
+    size_t get_last_glyph_index() const
     {
         return glyphs.size() - 1;
     }
-    size_t get_num_glyphs()
+    size_t get_num_glyphs() const
     {
         return glyphs.size();
     }
-    long get_hinting_factor()
+    long get_hinting_factor() const
     {
         return hinting_factor;
     }
+    FT_Bool has_kerning() const
+    {
+        return FT_HAS_KERNING(face);
+    }
 
   private:
+    WarnFunc ft_glyph_warn;
     FT2Image image;
     FT_Face face;
     FT_Vector pen;    /* untransformed origin  */
     std::vector<FT_Glyph> glyphs;
+    std::vector<FT2Font *> fallbacks;
+    std::unordered_map<FT_UInt, FT2Font *> glyph_to_font;
+    std::unordered_map<long, FT2Font *> char_to_font;
     FT_BBox bbox;
     FT_Pos advance;
     long hinting_factor;
